@@ -46,6 +46,7 @@
 #include "math.h"
 #include "dact_common.h"
 #include "algorithms.h"
+#include "libdact.h"
 #include "ciphers.h"
 #include "module.h"
 #include "header.h"
@@ -62,13 +63,10 @@
 
 int print_help(int argc, char **argv);
 int dact_shutdown(int retval);
-char *dact_getoutfilename(const char *orig, const int mode);
-uint32_t dact_process_other(int src, const int dest, const uint32_t magic, const char *options);
 int main(int argc, char **argv);
 
 extern char *optarg;
 extern int optind, opterr, optopt;
-char dact_nonetwork=0;
 
 int print_help(int argc, char **argv) {
 	printf("DACT %i.%i.%i-%s by Keene Enterprises <dact@rkeene.org>\n", DACT_VER_MAJOR, DACT_VER_MINOR, DACT_VER_REVISION, DACT_VER_SUB);
@@ -92,6 +90,7 @@ int print_help(int argc, char **argv) {
 	printf("  -V          Display DACT version (%i.%i.%i-%s).\n", DACT_VER_MAJOR, DACT_VER_MINOR, DACT_VER_REVISION, DACT_VER_SUB);
 	printf("  -N          Upgrade DACT.\n");
 	printf("  -a          Upgrade DACT modules.\n");
+	printf("  -x          Create self-extracting DACT file.\n");
 	printf("  -b NN       Use a block size of NN bytes for compression.\n");
 	printf("  -e NN       Exclude algorithm NN from being used.\n");
 	printf("  -m CONF     Load config file CONF.\n");
@@ -371,26 +370,20 @@ int dact_shutdown(int retval) {
 	return(retval);
 }
 
-char *dact_getoutfilename(const char *orig, const int mode) {
+char *dact_getoutfilename(const char *orig, const int mode, const char *ext) {
 	char *ret=NULL;
-	int x=0;
+	size_t retlen;
 
 	switch (mode) {
 		case DACT_MODE_COMPR:
-			ret=malloc(strlen(orig)+5);
-#ifdef GO32
-			strncpy(ret,orig,8);
-			for (x=0;x<strlen(ret);x++) {
-				if (ret[x]=='.') ret[x]='_';
-			}
-			strcat(ret,".dct");
-#else
-			strcpy(ret,orig);
-			strcat(ret,".dct");
-#endif
+			retlen=strlen(orig)+strlen(ext)+10;
+			ret=malloc(retlen);
+			snprintf(ret, retlen, "%s%s", orig, ext);
 			break;
 		case DACT_MODE_DECMP:
 			if (strcmp(&orig[strlen(orig)-4],".dct") && \
+				strcmp(&orig[strlen(orig)-4], ".exe") && \
+				strcmp(&orig[strlen(orig)-4], ".bin") && \
 				strcmp(&orig[strlen(orig)-4], ".bz2") && \
 				strcmp(&orig[strlen(orig)-5], ".tbz2") && \
 				strcmp(&orig[strlen(orig)-4], ".tgz") && \
@@ -398,10 +391,8 @@ char *dact_getoutfilename(const char *orig, const int mode) {
 				return(NULL);
 			}
 /* XXX: I wonder if this breaks easily... */
-			x=(strrchr(orig,'.')-orig);
-			ret=malloc(x+1);
-			strncpy(ret,orig,x);
-			ret[x]=0;
+			ret=strdup(orig);
+			(*strrchr(ret, '.'))='\0';
 			break;
 		case DACT_MODE_STAT:
 			return(NULL);
@@ -410,113 +401,22 @@ char *dact_getoutfilename(const char *orig, const int mode) {
 	return(ret);
 }
 
-uint32_t dact_process_other(int src, const int dest, const uint32_t magic, const char *options) {
-	char *buf, tmpbuf[128]="/tmp/dactXXXXXX";
-	uint32_t filesize=0, x;
-	int tmpfd=0;
-#if defined(HAVE_LIBBZ2) && (defined(HAVE_BZDOPEN) || defined(HAVE_NEW_BZDOPEN))
-	BZFILE *bzfd;
-#endif
-#if defined(HAVE_LIBZ) && defined(HAVE_GZDOPEN)
-	gzFile gzfd;
-#endif
-
-	filesize=0;  /* Fix a warning, that is all. */
-
-/*
- * bad and broke stuff XXX FIXME XXX FIXME XXX FIXME
- * There has to be a better way to do this... I just want
- * to rewind my socket/pipe 4 bytes... 4 bytes is all I ask
- * Is that so much to ask for?  Well?!  Is it?!  I don't
- * think it is.
- * I give up on this.
- *
- * Someone please fix it.
- *  -- Roy Keene <dact@rkeene.org>
- */
-	if (lseek_net(src, 0, SEEK_SET)<0) {
-/*
- * lseek_net() should make this obsolete.
- *  ... EXCEPT! when reading from stdin.
- */
-		tmpfd=mkstemp(tmpbuf);
-		write_de(tmpfd, magic, 4);
-		buf=malloc(1024);
-		while (1) {
-			x=read_f(src, buf, 1024);
-			write(tmpfd, buf, x);
-			if (x<1024) break;
-		}
-		close(src);
-		src=tmpfd;
-		lseek_net(src, 0, SEEK_SET); /* Now bitch. */
-		free(buf);
-	}
-#if defined(HAVE_LIBZ) && defined(HAVE_GZDOPEN)
-
-	if ((magic&0xffff0000)==0x1f8b0000) { /* gzip */
-		dact_ui_status(DACT_UI_LVL_GEN, "Gunzipping...");
-		buf=malloc(1024);
-
-		gzfd=gzdopen(src, "r");
-/*XXX: need to dact_ui_setup() */
-		while (1) {
-			dact_ui_incrblkcnt(1);
-			x=gzread(gzfd,buf,1024);
-			filesize+=write(dest, buf, x);
-			if (x<1024) break;
-		}
-		free(buf);
-		if (tmpfd!=0) unlink(tmpbuf);
-		return(filesize);
-	}
-#endif
-
-#if defined(HAVE_LIBBZ2) && (defined(HAVE_BZDOPEN) || defined(HAVE_NEW_BZDOPEN))
-	if ((magic&0xffffff00)==0x425a6800) { /* bzip2 */ 
-		dact_ui_status(DACT_UI_LVL_GEN, "Bunzipping...");
-		buf=malloc(1024);
-
-#ifdef HAVE_NEW_BZDOPEN
-		bzfd=BZ2_bzdopen(src, "r");
-#else
-		bzfd=bzdopen(src, "r");
-#endif
-/*XXX: need to dact_ui_setup() */
-		while(1) {
-			dact_ui_incrblkcnt(1);
-#ifdef HAVE_NEW_BZDOPEN
-			x=BZ2_bzread(bzfd, buf, 1024);
-#else
-			x=bzread(bzfd, buf, 1024);
-#endif
-			filesize+=write(dest, buf, x);
-			if (x<1024) break;
-		}
-		free(buf);
-		if (tmpfd!=0) unlink(tmpbuf);
-		return(filesize);
-	}
-#endif
-	return(0);
-}
-
 int main(int argc, char **argv) {
-	unsigned char options[20]={0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
+	unsigned char options[20]={0};
 	signed char opt;
 	struct stat stat_buf;
 	char **in_files, *in_file=NULL, *out_file=NULL;
 	char dact_binfilebuf[256], *dact_binfile;
+	char *ext=".dct";
 	int filecnt=0;
 	int in_fd, out_fd;
 	int mode=DACT_MODE_COMPR, ciphernum=-1;
 	uint32_t dact_blk_size=0;
-	uint32_t crcs[6]={0,0,0,0,0,0};
+	uint32_t crcs[6]={0};
 	uint32_t i,x;
 
-
 	dact_ui_init();
-
+	dact_init();
 
 /* hack, to make upgrade work even if DACT_OPT_BINCHK is enabled, we must
  *      get the new version before executing it.
@@ -551,7 +451,7 @@ int main(int argc, char **argv) {
 		}
 	}
 
-	while ((opt=getopt(argc,argv,"adfsvcnhiNVHCM:E:p:I:m:e:lb:u:U:TPOD:o:"))!=-1) {
+	while ((opt=getopt(argc,argv,"adfsvcnhixNVHCM:E:p:I:m:e:lb:u:U:TPOD:o:"))!=-1) {
 		switch (opt) {
 			case 'a':
 				options[DACT_OPT_UPGRADE]=1;
@@ -568,6 +468,14 @@ int main(int argc, char **argv) {
 			case 'i':
 				dact_ui_setopt(DACT_UI_OPT_PASSSTDIN, 1);
 				break;
+			case 'x':
+				if (strcmp(EXEEXT, "")==0) {
+					ext=".bin";
+				} else {
+					ext=EXEEXT;
+				}
+				options[DACT_OPT_SFX]=!options[DACT_OPT_SFX];
+				break;
 			case 'c':
 				options[DACT_OPT_STDOUT]=!options[DACT_OPT_STDOUT];
 				break;
@@ -718,7 +626,7 @@ int main(int argc, char **argv) {
 		in_file=in_files[filecnt];
 		if (in_file!=NULL) {
 /* Determine resulting file name */
-			if (out_file==NULL) out_file=dact_getoutfilename(in_file,mode);
+			if (out_file==NULL) out_file=dact_getoutfilename(in_file, mode, ext);
 			if (strcmp("-",in_file)==0) {
 				in_fd=STDIN_FILENO;
 			} else {
